﻿Imports VirtualPhotoFrame.ServiceReference
Imports Microsoft.WindowsLive.Id.Client
Imports System.Net
Imports System.ServiceModel
Imports System.ServiceModel.Channels
Imports System.ServiceModel.Syndication
Imports System.Threading
Imports System.Windows.Threading

' ソースコードの利用は、自由に使用して頂いて構いません。

Class FrameWindow

    ' 表示するコレクションをメニューから選択したときに実行するコマンド
    Public Shared ReadOnly SelectCollectionCommand As RoutedCommand = _
        New RoutedCommand("SelectCollection", GetType(FrameWindow))

    ' Windows Live ID Client 用
    Private Manager As IdentityManager
    Private Identity As Identity

    ' Web サービスの URL
    Private ServiceUrl As String = "https://frameit.live.com/service/devicesvc.asmx"

    ''' <summary>
    ''' 登録トークンを使用したデバイスの登録
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub RegisterDeviceWithToken()

        Dim result As DeviceBindResults
        Using client = New DeviceSvcSoapClient
            Dim manufacturerId = "Virtual Photo Frame" '製造元の名前
            Dim serialNumber = Now.Ticks.ToString ' シリアル番号

            ' 登録トークンの取得
            Dim tokenResult = client.GetClaimToken(manufacturerId, serialNumber)
            If tokenResult.ResponseCode <> 0 Then
                ' (ResponseCode = 1 のとき取得失敗)
                Exit Sub
            End If

            ' ユーザーにトークンの提示
            MessageBox.Show(tokenResult.ClaimUrl & " へアクセスして登録トークン " & _
                            tokenResult.ClaimToken & " を設定してください。" & vbCrLf & _
                            "設定完了後 OK ボタンをクリックしてください。")

            ' デバイスIDの取得
            result = client.DeviceBind(tokenResult.ClaimToken, manufacturerId, serialNumber)
        End Using

        If result.ResponseCode = 0 Then
            ' 取得成功 (ResponseCode = 0)

            ' デバイスIDの保存例 (プロジェクトのプロパティの設定で String型の DeviceId という値を追加しておきます)
            My.Settings.DeviceId = result.DeviceId
            My.Settings.Save()
            MessageBox.Show("登録完了しました。")

            ' コレクションの取得
            GetCollectionInfo()
        End If
    End Sub

    ''' <summary>
    ''' デバイスの登録
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub RegisterDevice()

        Try
            ' IdentityManager, Identity インスタンス生成
            Manager = IdentityManager.CreateInstance("VirtualPhotoFrame;***@hotmail.com;Virtual Photo Frame", "Virtual Photo Frame")
            Identity = Manager.CreateIdentity
        Catch ex As WLLogOnException
            ' 生成失敗
            MessageBox.Show(ex.ErrorMessage)
            Exit Sub
        End Try

        If Not Identity.IsAuthenticated Then
            ' 認証されていない場合

            ' ダイアログボックスの表示
            If Identity.Authenticate Then
                ' 認証が成功した場合
                ' (ユーザー名(メールアドレス)などが取得可能)
                Dim userName = Identity.UserName
            Else
                ' 認証キャンセル
                Exit Sub
            End If
        End If

        ' チケットの取得
        Dim ticket As String
        Try
            ticket = Identity.GetTicket(ServiceUrl, "MBI", True)
        Catch ex As WLLogOnException
            ' 取得失敗
            MessageBox.Show(ex.ErrorMessage)
            Exit Sub
        End Try

        ' Live FrameIt API とのやり取り
        Dim result As DeviceBindResults
        Using client = New DeviceSvcSoapClient
            Using scope = New OperationContextScope(client.InnerChannel)
                ' HTTP ヘッダーフィールドの追加
                Dim prop = New HttpRequestMessageProperty
                prop.Headers.Add(Net.HttpRequestHeader.Authorization, "WLID1.0 " & ticket)
                OperationContext.Current.OutgoingMessageProperties.Add(HttpRequestMessageProperty.Name, prop)

                ' デバイスの登録
                result = client.DeviceBindUser("Virtual Photo Frame", Now.Ticks.ToString)
            End Using
        End Using

        If result.ResponseCode = 0 Then
            ' 取得成功 (ResponseCode = 0)

            ' デバイスIDの保存 (プロジェクトのプロパティの設定で String型の DeviceId という値を追加しておきます)
            My.Settings.DeviceId = result.DeviceId
            My.Settings.Save()
            MessageBox.Show("登録完了しました。")

            ' コレクションの取得
            GetCollectionInfo()
        End If
    End Sub


    ''' <summary>
    ''' コレクション情報の取得、メニューへ反映
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub GetCollectionInfo()

        ' Live FrameIt とのやり取り
        Using client = New DeviceSvcSoapClient
            Dim results = client.GetCollectionInfo(My.Settings.DeviceId)

            If results.ResponseCode <> 0 Then
                MessageBox.Show("コレクションの取得に失敗しました。")
                Exit Sub
            End If

            ' コレクションをメニューに設定
            CollectionMenuItem.ItemsSource = results.CollectionInfoList
        End Using

    End Sub


    ' ダウンロードした画像を保存するフォルダパス
    Private CachePath As String = System.IO.Path.Combine(My.Application.Info.DirectoryPath, "cache")

    ''' <summary>
    ''' ウィンドウロード時処理
    ''' フォルダ作成と、デバイス ID を持っていれば各種情報の取得
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FrameWindow_Loaded(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles Me.Loaded
        ' 画像をダウンロードするフォルダ作成
        My.Computer.FileSystem.CreateDirectory(CachePath)
        If My.Settings.DeviceId = "" Then
            Exit Sub
        End If

        ' コレクションの取得、メニュー設定
        GetCollectionInfo()

        ' RSS フィード 読み出し
        If My.Settings.CollectionFeedUrl <> "" Then
            Dim thread = New System.Threading.Thread(AddressOf FetchFeed)
            thread.Start(My.Settings.CollectionFeedUrl)
        End If
    End Sub

    Private Sub Window_MouseLeftButtonDown(ByVal sender As System.Object, ByVal e As System.Windows.Input.MouseButtonEventArgs)
        ' ウィンドウをドラッグで移動できるようにする
        DragMove()
    End Sub

    Private Sub RegisterMenuItem_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        ' デバイスの登録
        RegisterDevice()
    End Sub

    Private Sub GetCollectionMenuItem_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        ' コレクションの取得
        GetCollectionInfo()
    End Sub

    Private Sub ExitMenuItem_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs)
        ' アプリケーションの終了
        Me.Close()
    End Sub

    ''' <summary>
    ''' コマンドの実行
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SelectCollectionCommand_Executed(ByVal sender As System.Object, ByVal e As System.Windows.Input.ExecutedRoutedEventArgs)
        ' コレクションの選択

        ' コレクション RSS フィード URL の保存
        Dim url = CStr(e.Parameter)
        My.Settings.CollectionFeedUrl = url
        My.Settings.Save()

        ' RSS フィード 取得
        Dim thread = New System.Threading.Thread(AddressOf FetchFeed)
        thread.Start(url)
    End Sub

    ' RSS フィード更新用タイマー
    Private WithEvents FeedTimer As New DispatcherTimer

    ' スライドショー(画像切り替え)表示用タイマー
    Private WithEvents SlideShowTimer As New DispatcherTimer

    ' 画像 URL から保存したファイル名を参照するための Dictionary
    Private CachedImages As New Dictionary(Of String, String)
    Private CachedImagesLock As New ReaderWriterLockSlim

    ' スライドショー用の List, 表示順に 画像 URL を格納
    Private SlideShowImageUrls As New List(Of String)
    Private SlideShowIndex As Integer
    Private SlideShowImageUrlsLock As New ReaderWriterLockSlim

    ''' <summary>
    ''' RSS フィードの取得
    ''' </summary>
    ''' <param name="state"></param>
    ''' <remarks></remarks>
    Private Sub FetchFeed(ByVal state As Object)

        FeedTimer.Stop()
        SlideShowTimer.Stop()

        ' RSS フィード取得
        Dim url = CStr(state) ' RSS フィード URL
        Dim feed As SyndicationFeed
        Try
            feed = SyndicationFeed.Load(Xml.XmlReader.Create(url))
        Catch ex As Exception
            ' 取得できなかった場合、時間を置いて再度取得する
            FeedTimer.Interval = New TimeSpan(0, 1, 0)
            FeedTimer.Start()
            SlideShowTimer.Start()
            Exit Sub
        End Try

        ' 有効時間(分)取得
        Dim ttl = (From ext In feed.ElementExtensions _
                   Where ext.OuterName = "ttl" _
                   Select ext.GetObject(Of Integer)()).SingleOrDefault

        ' フィードの発行日時取得
        Dim pubDate = (From ext In feed.ElementExtensions _
                       Where ext.OuterName = "pubDate" _
                       Select ext.GetObject(Of String)()).SingleOrDefault

        ' 次回フィード取得時間を求めタイマーを設定
        Dim span = CDate(pubDate).AddMinutes(ttl).Subtract(Now) ' 次回フィードするまでの時間
        FeedTimer.Interval = span
        FeedTimer.Start()

        ' スライドショーの URL リストをクリア
        SlideShowImageUrlsLock.EnterWriteLock()
        SlideShowImageUrls.Clear()
        SlideShowImageUrlsLock.ExitWriteLock()

        ' フィードから画像 URL 参照
        Dim newImages = New List(Of String)
        For Each item In feed.Items
            ' <media:content> の url 属性値取得
            Dim imageUrl = (From ext In item.ElementExtensions _
                            Where ext.OuterName = "content" _
                            AndAlso ext.OuterNamespace = "http://search.yahoo.com/mrss/" _
                            Select ext.GetReader.GetAttribute("url")).SingleOrDefault

            ' 取得順にリストへ画像 URL を追加
            SlideShowImageUrlsLock.EnterWriteLock()
            SlideShowImageUrls.Add(imageUrl)
            SlideShowImageUrlsLock.ExitWriteLock()

            newImages.Add(imageUrl)
        Next

        ' フィードにない画像ファイル削除
        Dim deletedKeys = New List(Of String)
        For Each k In CachedImages.Keys
            If newImages.Contains(k) Then
                Continue For
            End If
            Try
                CachedImagesLock.EnterReadLock()
                ' TODO 削除できない場合、事後対応
                My.Computer.FileSystem.DeleteFile(CachedImages(k))
                deletedKeys.Add(k)
            Catch ex As Exception
                ' Ignore
            Finally
                CachedImagesLock.ExitReadLock()
            End Try
        Next

        CachedImagesLock.EnterWriteLock()
        For Each k In deletedKeys
            CachedImages.Remove(k)
        Next
        CachedImagesLock.ExitWriteLock()

        ' 新しい画像をダウンロード
        For Each u In newImages

            CachedImagesLock.EnterReadLock()
            Dim contains = CachedImages.ContainsKey(u)
            CachedImagesLock.ExitReadLock()
            If contains Then
                Continue For
            End If

            ' 新しくダウンロード
            Try
                Dim client = New WebClient
                Dim file = System.IO.Path.Combine(CachePath, Guid.NewGuid.ToString)
                client.DownloadFile(u, file)
                CachedImagesLock.EnterWriteLock()
                CachedImages.Add(u, file)
                CachedImagesLock.ExitWriteLock()
            Catch ex As Exception
                ' Ignore
            End Try
        Next

        ' スライドショーを最初から表示
        System.Threading.Interlocked.Exchange(SlideShowIndex, 0)

        ' スライドショータイマーの設定
        SlideShowTimer.Interval = New TimeSpan(0, 0, 0) ' すぐ表示するため Interval を 0 に設定
        SlideShowTimer.Start()

    End Sub

    ''' <summary>
    ''' RSS フィード取得用のタイマー処理
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FeedTimer_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles FeedTimer.Tick
        FeedTimer.Stop()

        ' RSS フィード 再取得
        Dim thread = New System.Threading.Thread(AddressOf FetchFeed)
        thread.Start(My.Settings.CollectionFeedUrl)
    End Sub

    ''' <summary>
    ''' スライドショー用のタイマー処理
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SlideShowTimer_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles SlideShowTimer.Tick
        SlideShowTimer.Stop()

        ' 表示する画像の URL を List から取得
        Dim url As String
        Dim count As Integer
        SlideShowImageUrlsLock.EnterReadLock()
        url = SlideShowImageUrls(SlideShowIndex)
        count = SlideShowImageUrls.Count
        SlideShowImageUrlsLock.ExitReadLock()

        ' ダウンロードした画像があるかどうか
        Dim exists As Boolean
        Dim file As String = ""
        CachedImagesLock.EnterReadLock()
        If CachedImages.ContainsKey(url) Then
            file = CachedImages(url)
            exists = System.IO.File.Exists(file)
        Else
            exists = False
        End If
        CachedImagesLock.ExitReadLock()

        ' 画像がある場合、Image コントロールに設定
        If exists Then
            Try
                PhotoImage.Source = New BitmapImage(New Uri(file))
            Catch ex As Exception
            End Try
        End If

        ' List の Index 値をインクリメントまたは 0 に戻す
        Dim prevIndex = Interlocked.Increment(SlideShowIndex)
        If prevIndex >= count - 1 Then
            Interlocked.Exchange(SlideShowIndex, 0)
        End If

        ' 次回画像表示用にタイマーを設定
        SlideShowTimer.Interval = New TimeSpan(0, 0, 10)
        SlideShowTimer.Start()
    End Sub
End Class
